/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package j8086emu.model.hardware;

import j8086emu.model.exeptions.RamOutOfRangeExeption;
import j8086emu.model.hardware.ram.Ram;
import j8086emu.model.hardware.registers.*;
import j8086emu.model.interfaces.Variable;
import j8086emu.model.interfaces.Widgetable;
import j8086emu.model.progcode.InstructionsMap;
import j8086emu.model.progcode.types.VarTypes;
import j8086emu.view.GUI;
import j8086emu.view.TextWidget;

import java.util.HashMap;

/**
 * @author Sone
 */
public class CPU extends HashMap<String, Object> {

    private GUI gui;
    private InstructionsMap instructionsMap;

    public GUI getGui() {
        return gui;
    }

    public InstructionsMap getInstructionsMap() {
        return instructionsMap;
    }

    public void init(GUI gui) {
        this.gui = gui;
    }

    public CPU() {
        initHardware();
        instructionsMap = new InstructionsMap(this);
    }

    public void reset() {
        setRegData("AX", 0);
        setRegData("BX", 0);
        setRegData("CX", 0);
        setRegData("DX", 0);
        setRegData("SI", 0);
        setRegData("DI", 0);
        setRegData("BP", 0);
        setRegData("SP", 0xffff);
        setRegData("CS", 0);
        setRegData("DS", 0);
        setRegData("ES", 0);
        setRegData("SS", 0);
        setRegData("IP", 0);
        setFlagState("CF", false);
        setFlagState("PF", false);
        setFlagState("AF", true);
        setFlagState("ZF", false);
        setFlagState("SF", false);
        setFlagState("TF", false);
        setFlagState("IF", false);
        setFlagState("DF", false);
        setFlagState("OF", false);
        resetRam();
    }

    public void setRegData(String name, int data) {
        Variable reg;
        reg = (Variable) get(name);
        if (reg != null) {
            reg.setData(data);
        } else {
            throw new IllegalArgumentException(name + " not such Reg");
        }

    }

    public int getRegData(String name) {
        Variable reg;
        reg = (Variable) get(name);
        if (reg == null) {
            throw new IllegalArgumentException(name + " not such Reg");
        } else {
            return reg.getData();
        }
    }

    public void setRegWidget(RegNames regName, TextWidget widget) {
        ((Widgetable) get(regName.toString())).setWidget(widget);
    }

    public void setRamData(int cellNum, int data) throws RamOutOfRangeExeption {
        Ram ram = (Ram) get("Ram");
        ram.setByte(cellNum, data);
    }

    public int getRamData(int cellNum) {
        Ram ram = (Ram) get("Ram");
        return ram.getByte(cellNum);
    }

    public int getRamDataUnsafe(int cellNum) {
        Ram ram = (Ram) get("Ram");
        return ram.getByteUnsafe(cellNum);
    }

    public Boolean getFlagState(String flagName) {
        return ((FlagsRegister) (get("FLAGS"))).getFlag(flagName);
    }

    public void setFlagState(String flagName, boolean state) {
        ((FlagsRegister) (get("FLAGS"))).setFlag(flagName, state);
    }

    private void initHardware() {
        GeneralRegister ax = new GeneralRegister("AX");
        GeneralRegister bx = new GeneralRegister("BX");
        GeneralRegister cx = new GeneralRegister("CX");
        GeneralRegister dx = new GeneralRegister("DX");

        this.put("AX", ax);
        this.put("BX", bx);
        this.put("CX", cx);
        this.put("DX", dx);
        this.put("AH", new RegHighBits(ax));
        this.put("BH", new RegHighBits(bx));
        this.put("CH", new RegHighBits(cx));
        this.put("DH", new RegHighBits(dx));
        this.put("AL", new RegLowBits(ax));
        this.put("BL", new RegLowBits(bx));
        this.put("CL", new RegLowBits(cx));
        this.put("DL", new RegLowBits(dx));
        this.put("SI", new SimpleRegister("SI", 0, VarTypes.reg16));
        this.put("DI", new SimpleRegister("DI", 0, VarTypes.reg16));
        this.put("BP", new SimpleRegister("BP", 0, VarTypes.reg16));
        this.put("SP", new SimpleRegister("SP", 0xffff, VarTypes.reg16));
        this.put("CS", new SimpleRegister("CS", 0, VarTypes.segreg));//Segment registers^
        this.put("DS", new SimpleRegister("DS", 0, VarTypes.segreg));//                 ^
        this.put("ES", new SimpleRegister("ES", 0, VarTypes.segreg));//                 ^
        this.put("SS", new SimpleRegister("SS", 0, VarTypes.segreg));//                 ^
        this.put("IP", new SimpleRegister("IP", 0, VarTypes.reg16));//Instruction Pointer
        this.put("FLAGS", new FlagsRegister());
        this.put("Ram", new Ram());
    }

    private void resetRam() {
        ((Ram) get("Ram")).reset();
    }

}
