package j8086emu.model.progcode;

import j8086emu.model.exeptions.IncorrectInstructionException;
import j8086emu.model.exeptions.InvalidArgumentExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.interfaces.Performable;

import java.util.HashMap;

/**
 * @author vlad
 *         Date: 10.03.2010
 *         Time: 16:53:36
 */
public class CommandBuilder {
    //private static final int DEFAULT_COMMAND_LENGTH = 3;
    private InstructionsMap instructionsMap;
    private final String[] instructionPatterns;
    private final String dataPattern = "(.+ )?D[BW] .+\n";
    private final String labelPattern = ".+:\n";
    private VarFinder varFinder;
    private LabelFinder labelFinder;

    public CommandBuilder(CPU cpu) {
        instructionsMap = cpu.getInstructionsMap();
        instructionPatterns = instructionsMap.keyArray();
        this.varFinder = instructionsMap.getVarFinder();
        this.labelFinder = instructionsMap.getLabelFinder();
    }

    public HashMap<Integer, Command> createCommands(String[] code, int segAddress) throws IncorrectInstructionException {
        //Command[] commands = new Command[code.length];
        //int commandOffset = 0;
        HashMap<Integer, Command> commandsMap = new HashMap<Integer, Command>();
        int commandOffset = 0;
        int commandFullAddress = calculateFullAddress(segAddress, commandOffset);
        int lastCommandLength;
        boolean lastIsCommand = true;

        for (int i = 0; i < code.length; i++) {
            String currentLine = code[i];
            String currentPattern = "";
            int concurrencesNum = 0;
            String instructionArgs;

            if (i > 0 && lastIsCommand) {
                lastCommandLength = commandsMap.get(commandFullAddress).getLength();
                commandOffset += lastCommandLength;
                commandFullAddress += lastCommandLength;
            }

            if (currentLine.matches(labelPattern)) {
                labelFinder.allocateLabel(currentLine, commandOffset);
                lastIsCommand = false;
                continue;
            }

            if (currentLine.matches(dataPattern)) {
                try {
                    Command varDefine = buildVariable(currentLine, commandFullAddress);
                    commandsMap.put(commandFullAddress, varDefine);
                    lastIsCommand = true;
                } catch (InvalidArgumentExeption invalidArgumentExeption) {
                    invalidArgumentExeption.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }
                continue;
            }

            for (String pattern : instructionPatterns) {
                if (currentLine.matches(pattern)) {
                    concurrencesNum++;
                    currentPattern = pattern;
                }
            }
            if (concurrencesNum != 1) {
                throw new IncorrectInstructionException(i);
            }
            // If instruction string is not contain 2 words, separated by a space when it's instruction without arguments
            if (!currentLine.matches(".+ .+\n")) {
                instructionArgs = null;
            } else {
                // Getting argument pattern from instruction pattern
                instructionArgs = currentLine.replaceFirst(".+ ", "");
            }
            // perform instruction with string arguments
            Performable currentInstruct = instructionsMap.get(currentPattern);
            if (currentInstruct == null) {
                throw new IncorrectInstructionException(i);
            }
            commandsMap.put(commandFullAddress, new Command(currentInstruct, instructionArgs));
            lastIsCommand = true;
        }
        return commandsMap;
    }

    private int calculateFullAddress(int segAddress, int offset) {
        segAddress = segAddress << 4;
        return segAddress + offset;
    }

    private Command buildVariable(String defineStr, int varAddress) throws InvalidArgumentExeption {
        int varLength = varFinder.allocateVariable(defineStr, varAddress);
        return new Command(varLength);
    }
}
