package j8086emu.model.progcode;

import j8086emu.model.hardware.CPU;
import j8086emu.model.interfaces.Performable;
import j8086emu.model.progcode.instructions.*;

import java.util.HashMap;
import java.util.Iterator;

/**
 * @author vlad
 */
public class InstructionsMap {

    private final HashMap<String, Performable> instructions = new HashMap<String, Performable>();
    //private HashMap<String, Integer> labelsMap = new HashMap<String, Integer>();

    public VarFinder getVarFinder() {
        return varFinder;
    }

    private VarFinder varFinder;
    private LabelFinder labelFinder;
    private Int i;

    // TODO повторяющиеся объекты заменить ссылками

    public InstructionsMap(CPU cpu) {
        NameMap nm = new NameMap(cpu);
        varFinder = new VarFinder(nm);
        VarParser varParser = new VarParser(nm);
        labelFinder = new LabelFinder(nm);
        instructions.put("[Mm][Oo][Vv] \\S+,\\S+\n", new Mov(varParser));
        instructions.put("[Aa][Dd][Dd] \\S+,\\S+\n", new Add(varParser));
        instructions.put("[Ss][Uu][Bb] \\S+,\\S+\n", new Sub(varParser));
        instructions.put("[Dd][Ee][Cc] \\w+\n", new Dec(varParser));
        instructions.put("[Ii][Nn][Cc] \\w+\n", new Inc(varParser));
        instructions.put("[Aa][Nn][Dd] \\w+,\\w+\n", new And(varParser));
        instructions.put("[Oo][Rr] \\w+,\\w+\n", new Or(varParser));
        instructions.put("[Xx][Oo][Rr] \\w+,\\w+\n", new Xor(varParser));
        instructions.put("[Ss][Aa][Rr] \\w+,\\w+\n", new Sar(varParser));
        instructions.put("[Ss][Aa][Ll] \\w+,\\w+\n", new ShlSal(varParser));
        instructions.put("[Ss][Hh][Ll] \\w+,\\w+\n", new ShlSal(varParser));
        instructions.put("[Ss][Hh][Rr] \\w+,\\w+\n", new Shr(varParser));
        instructions.put("[Pp][Uu][Ss][Hh] \\w+\n", new Push(varParser));
        instructions.put("[Pp][Oo][Pp] \\w+\n", new Pop(varParser));
        instructions.put("[Cc][Bb][Ww]\n", new Cbw(varParser));
        instructions.put("[Cc][Ww][Dd]\n", new Cwd(varParser));
        instructions.put("[Mm][Uu][Ll] \\w+\n", new Mul(varParser));
        instructions.put("[Ii][Mm][Uu][Ll] \\w+\n", new Imul(varParser));
        instructions.put("[Dd][Ii][Vv] \\w+\n", new Div(varParser));
        instructions.put("[Nn][Ee][Gg] \\w+\n", new Neg(varParser));
        instructions.put("[Nn][Oo][Tt] \\w+\n", new Not(varParser));
        instructions.put("[Cc][Mm][Pp] \\S+,\\S+\n", new Cmp(varParser));
        instructions.put("[Jj][EeZz] \\w+\n", new JeJz(varParser));
        instructions.put("[Jj]([Gg][Ee]|[Nn][Ll]) \\w+\n", new JgeJnl(varParser));
        instructions.put("[Jj]([Gg]|[Nn][Ll][Ee]) \\w+\n", new JgJnle(varParser));
        instructions.put("[Jj]([Ll][Ee]|[Nn][Gg]) \\w+\n", new JleJng(varParser));
        instructions.put("[Jj]([Ll]|[Nn][Gg][Ee]) \\w+\n", new JlJnge(varParser));
        instructions.put("[Jj][Nn][EeZz] \\w+\n", new JneJnz(varParser));
        instructions.put("[Jj][Nn][Oo] \\w+\n", new Jno(varParser));
        instructions.put("[Jj]([Nn][Pp]|[Pp][Oo]) \\w+\n", new JnpJpo(varParser));
        instructions.put("[Jj][Nn][Ss] \\w+\n", new Jns(varParser));
        instructions.put("[Jj][Oo] \\w+\n", new Jo(varParser));
        instructions.put("[Jj][Pp][Ee]? \\w+\n", new JpJpe(varParser));
        instructions.put("[Jj][Ss] \\w+\n", new Js(varParser));
        //instructions.put("[Aa][Ss][Ss][Uu][Mm][Ee] [CcDdSsEe][Ss]:.+\n", new Assume());
        i = new Int(varParser);
        instructions.put("[Ii][Nn][Tt] \\w+\n", i);
    }
    //TODO push and pop tests

    public boolean containsKey(String str) {
        return instructions.containsKey(str);
    }

    public Performable get(String str) {
        return instructions.get(str);
    }

    public String[] keyArray() {
        // actions below is for type casting Object[] to String[]
        String[] instructPatterns = new String[instructions.size()];
        Iterator instructIterator = instructions.keySet().iterator();
        for (int i = 0; i < instructPatterns.length; i++) {
            instructPatterns[i] = (String) instructIterator.next();
        }
        return instructPatterns;
    }

    public LabelFinder getLabelFinder() {
        return labelFinder;
    }

    public void setRunProgramHandler(RunProgramHandler handler){
        i.setRunProgramHandler(handler);
    }

}
