package j8086emu.model.progcode;

import j8086emu.model.exeptions.*;
import j8086emu.model.hardware.CPU;
import j8086emu.view.GUI;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

/**
 * @author vlad
 */
public class RunProgramHandler {

    public static int DEFAULT_SEGMENT_ADDRESS = 15032; // any random address

    private final PointerCode pointerCode;
    private Segmentator segmentator;
    private Segment segment;
    private final CPU cpu;
    private GUI gui;
    private Disassembler disassembler;
    private HashMap<Integer, String> programMapForRam;


    private DosBoxLauncher dosBoxLauncher;

    public RunProgramHandler(ProgramCode code, PointerCode pointer, CPU cpu, GUI gui) {
        segmentator = new Segmentator(code, cpu);
        dosBoxLauncher = new DosBoxLauncher();
        disassembler = new Disassembler();
        programMapForRam = new HashMap<Integer, String>();
        pointerCode = pointer;
        this.cpu = cpu;
        this.gui = gui;
    }

    public boolean runStep() throws IncorrectInstructionException, InvalidArgumentExeption, PositiveOverflowExeption, NegativeOverflowExeption {
//        if(programCode.getCode().length == pointerCode.getStringNum()) {
//            return false;
//        }
//        codeTranslator.performCurrentInstruction();
        /*if (null == segment) {
            // TODO: warning! need to compile
            return false;
        }*/
        if (0 == pointerCode.getStringNum()) {
            cpu.reset();
            //compile();
            initProgram();
        }
        int offset = cpu.getRegData("IP");
        Command currentCommand = segment.getCommandAt(offset);
        if (null == currentCommand) {
            return false;
        }
        currentCommand.perform();
        if (!currentCommand.isJump()) {
            cpu.setRegData("IP", offset + currentCommand.getLength());
            pointerCode.nextString();
        }
        gui.reset();
        return true;
    }

    private void initProgram() {
        writeProgramInRam();
        initSegRegisters();
    }

    private void writeProgramInRam() {
        Iterator<Entry<Integer,String>> iterator = programMapForRam.entrySet().iterator();
        int address;
        int offset;
        String hexInstruction;
        while (iterator.hasNext()) {
            Entry<Integer,String> entry = iterator.next();
            offset = entry.getKey();
            hexInstruction = entry.getValue();
            String nextbyte;
            for(int i = 0, length = hexInstruction.length(); i < length; i+=2) {
                nextbyte = hexInstruction.substring(i, i + 1);
                address = calculateFullAddress(DEFAULT_SEGMENT_ADDRESS, offset + i/2);
                try {
                    cpu.setRamData(address, Integer.parseInt(nextbyte, 16));
                } catch (RamOutOfRangeExeption ramOutOfRangeExeption) {
                    ramOutOfRangeExeption.printStackTrace();
                }
            }
        }
    }

    private int calculateFullAddress(int startAddress, int offset) {
        int fullAddress = (startAddress << 4) + offset;
        return fullAddress;
    }

    private void initSegRegisters() {
        //int codeSegAddr = segment.getStartAddress();
        cpu.setRegData("CS", DEFAULT_SEGMENT_ADDRESS);
        cpu.setRegData("DS", DEFAULT_SEGMENT_ADDRESS);
        cpu.setRegData("ES", DEFAULT_SEGMENT_ADDRESS);
        cpu.setRegData("SS", DEFAULT_SEGMENT_ADDRESS);
    }

    public void runAllProgram() throws IncorrectInstructionException, InvalidArgumentExeption, PositiveOverflowExeption, NegativeOverflowExeption {
        pointerCode.reset();
        while (runStep()) {
        }
    }

    public void compile() {
        dosBoxLauncher.launchTasm();
//        segmentator.allocateCodeSegment();
//        segment = segmentator.getCodeSegment();
    }

    public void stopProgram() {

    }

    public void resumeProgram() {

    }

    public void disassembleProgram(ArrayList<InstructionStructure> machineInstructions) {
        int offset;
        for (int i = 0, size = machineInstructions.size(); i < size; i++) {
            offset = Integer.parseInt(machineInstructions.get(i).getOffset(), 16);
            programMapForRam.put(offset, machineInstructions.get(i).getHexInstruction());
            String disassambledInstruction = disassembler.disassemble(machineInstructions.get(i));
            machineInstructions.get(i).setStringInstruction(disassambledInstruction);
            // TODO get command object for execution from disassembler
        }
    }
}
