package j8086emu.model.progcode;

import j8086emu.model.exeptions.IncorrectInstructionException;
import j8086emu.model.hardware.CPU;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * @author vlad
 *         Date: 10.03.2010
 *         Time: 12:49:23
 */
public class Segmentator {

    public static int DEFAULT_SEGMENT_ADDRESS = 15032; // any random address

    private ProgramCode programCode;
    private CommandBuilder commandBuilder;
    private HashMap<String, Segment> segmentsMap;
    private int lastSegmentEndAddress;

    public Segmentator(ProgramCode code, CPU cpu) {
        segmentsMap = new HashMap<String, Segment>();
        commandBuilder = new CommandBuilder(cpu);
        this.programCode = code;
    }

    public void allocateSegments() {  // Keep code of this method on the future
//        resetFields();
//        String[] codeStrings = programCode.getCode();
//        String name;
//        int segStartIndex, segEndIndex = 0;
//        // TODO: show error when detect nested segments ??
//        for (int i = 0; i < codeStrings.length; i++) {
//            String codeString = codeStrings[i].toUpperCase();
//            if (codeString.matches(".+ SEGMENT\n")) { // TODO: show error if wrong segment name
//                name = codeString.replaceFirst(" SEGMENT", "").trim();
//                segStartIndex = i;
//                for (int j = segStartIndex + 1; j < codeStrings.length; j++) {
//                    if (codeStrings[j].matches(name + " ENDS\n")) {
//                        segEndIndex = j;
//                    } else {
//                        // TODO: show error: missing segment "ends"
//                    }
//                }
//                // allocate current segment body
//                String[] segStrings = new String[segEndIndex - segStartIndex - 1];
//                for (int k = segStartIndex + 1, count = 0; k < segEndIndex; k++, count++) {
//                    segStrings[count] = codeStrings[k];
//                }
//                Command[] commands = null;
//                try {
//                    commands = commandBuilder.createCommands(segStrings);
//                } catch (IncorrectInstructionException e) {
//                    e.printStackTrace();
//                }
//                createSegment(name, commands);
//                // continue searching after end of segment
//                i = segEndIndex;
//            }
//        }
//        if (segmentsMap.isEmpty()) {
//            // TODO: show error "segments not found"
//        }
    }

    public void allocateCodeSegment() {
        resetFields();
        String[] codeStrings = programCode.getCode();
        HashMap<Integer, Command> commandsMap = null;
        int segAddress = defineStartAddress();
        try {
            commandsMap = commandBuilder.createCommands(codeStrings, segAddress);
        } catch (IncorrectInstructionException e) {
            e.printStackTrace();
        }
        createSegment(".CODE", segAddress, commandsMap);
    }

    private void resetFields() {
        segmentsMap.clear();
        lastSegmentEndAddress = 0;
    }

    private void createSegment(String name, int startAddress, HashMap<Integer, Command> commandsMap) {
        if (null == commandsMap || commandsMap.isEmpty()) {
            // TODO: warning - empty segment
        } else {
            int lastCommandAddress = 0;
            Iterator addressIter = commandsMap.keySet().iterator();
            while(addressIter.hasNext()) {
                int addr = (Integer) addressIter.next();
                if(addr > lastCommandAddress) {
                    lastCommandAddress = addr;
                }
            }
            lastSegmentEndAddress = lastCommandAddress + commandsMap.get(lastCommandAddress).getLength();
            segmentsMap.put(name, new Segment(startAddress, commandsMap));
        }
    }

    private int defineStartAddress() {
        if (segmentsMap.isEmpty()) {
            return DEFAULT_SEGMENT_ADDRESS;
        } else {
            return lastSegmentEndAddress;
        }
    }

    public Segment getCodeSegment() {
        return segmentsMap.get(".CODE");
    }
}
