package j8086emu.model.progcode;

import j8086emu.model.exeptions.InvalidArgumentExeption;
import j8086emu.model.hardware.formats.GeneralRegisterDataFormat;
import j8086emu.model.interfaces.Variable;
import j8086emu.model.progcode.types.VarTypes;
import j8086emu.model.utils.DigitsParser;

import java.util.ArrayList;

/**
 * @author vlad
 *         Date: 23.03.2010
 *         Time: 18:49:27
 */
public class VarFinder {
    private DigitsParser digitParser;
    private VariableMap varMap;

    public VarFinder(NameMap nm) {
        varMap = nm.getVariableMap();
        digitParser = nm.getDigitsParser();
    }

    public int allocateVariable(String codeString, int varAddress) throws InvalidArgumentExeption {
        codeString = codeString.toUpperCase();  // TODO: Warning!!! Losing correct case of the string characters
        String name = null;
        String valueString;
        ArrayList<Integer> valueParts;
        String typeDirective = "DW";
        int partSize = 2;
        if (codeString.matches("(.+ )?DB .+\n")) { //TODO: checking for correct assembler name and not directive
            typeDirective = "DB";
            partSize = 1;
        }
        if (-1 != codeString.indexOf(" " + typeDirective + " ")) {  // If name exist before directive
            name = codeString.substring(0, codeString.indexOf(' '));
            valueString = codeString.substring(codeString.indexOf(" " + typeDirective + " ") + 4).trim();
        } else {
            valueString = codeString.substring(codeString.indexOf(typeDirective + " ") + 3).trim();    
        }
        valueParts = parseCorrectVariable(valueString, typeDirective.toLowerCase());

        int varPartAddress = varAddress;
        if ( ! valueParts.isEmpty()) {
            Variable var = varMap.addVar(name, VarTypes.valueOf(typeDirective.toLowerCase()), varAddress);
            var.setData(valueParts.get(0));
            name = null;
        }         
        for (int i = 1; i < valueParts.size(); i++) {
            varPartAddress += partSize;
            Variable var = varMap.addVar(name, VarTypes.valueOf(typeDirective.toLowerCase()), varPartAddress); // Address of var is number of string
            var.setData(valueParts.get(i));
        }
        return valueParts.size() * partSize;
    }

    private ArrayList<Integer> parseCorrectVariable(String valueString, String varType) throws InvalidArgumentExeption {
        String[] valueParts = valueString.split(",");
        ArrayList<Integer> correctParts = new ArrayList<Integer>();
        for (String part : valueParts) {
            if (part.matches("['\"].+['\"]")) { // Character and string variables
                for (int i = 1; i < part.length() - 1; i++) {
                    correctParts.add((int) part.charAt(i));    // TODO: if dw then each symbol presents in 16 bit
                }
            } else {
                int correctDigit = ((GeneralRegisterDataFormat) digitParser.parse(part, varType)).getData();
                correctParts.add(correctDigit);
            }
        }
        return correctParts;
    }
}