package j8086emu.model.progcode.instructions;

import j8086emu.model.exeptions.InvalidArgumentExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.interfaces.Performable;
import j8086emu.model.interfaces.Variable;
import j8086emu.model.progcode.VarParser;
import j8086emu.model.progcode.types.VarTypes;
import j8086emu.model.utils.Data;

/**
 * @author sone
 */
public class Add extends Performable {

    private final int NUM_OF_ARGS = 2;
    private final VarParser varParser;
    private Variable[] var;

    public Add(VarParser varParser) {
        this.varParser = varParser;
    }

    public void perform(String msg) throws InvalidArgumentExeption {
        var = varParser.getVar(msg, NUM_OF_ARGS);
        if (isValidTypes(var)) {
           /* if (var[1].getType() == VarTypes.db)
                if (var[0].getType() != VarTypes.reg8) {
                    if (var[1].getData() < 0) var[1].setData(0xFF00 | var[1].getData());

                    else var[1].setData(0xFF00 & var[1].getData());
                }*/
            CPU cpu = varParser.getCpu();
            String first, second;
            first = Data.intToHex(var[0].getData(),4);
            second = Data.intToHex(var[1].getData(),4);
            cpu.setFlagState("AF", Data.secondQuarterOverflow(first, second));
            cpu.setFlagState("CF", Data.borrowHighOrderBit(first, second));
            var[0].setData(var[0].getData() + var[1].getData());


            if ((var[0].getData() > 0xFFFF) || (var[0].getData() < 0)) {
                var[0].setData(Data.hexToInt(Integer.toHexString(var[0].getData()), 4));
                cpu.setFlagState("OF", true);
            } else cpu.setFlagState("OF", false);

            if (var[0].getData() < 0) cpu.setFlagState("SF", true);
            else cpu.setFlagState("SF", false);

            if (var[0].getData() == 0) cpu.setFlagState("ZF", true);
            else cpu.setFlagState("ZF", false);

            cpu.setFlagState("PF", Data.evenLowSetBits(Data.intToHex(var[0].getData(), 4)));

            
        } else {
            System.out.println(var[0].getType().toString() + " " + var[1].getType().toString());
            throw new InvalidArgumentExeption("Несоответствие типов");
        }
    }

    private boolean isValidTypes(Variable[] var) {
        if (isType0(VarTypes.reg16) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.mem))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.reg8))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.reg8))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.mem))
            return true;
        return false;
    }

    private boolean isType0(VarTypes type) {
        return var[0].getType() == type;
    }

    private boolean isType1(VarTypes type) {
        return var[1].getType() == type;
    }
}
