package j8086emu.model.progcode.instructions;

import j8086emu.model.exeptions.InvalidArgumentExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.interfaces.Performable;
import j8086emu.model.interfaces.Variable;
import j8086emu.model.progcode.VarParser;
import j8086emu.model.progcode.types.VarTypes;
import j8086emu.model.utils.Data;

/**
 * @author vlad
 */
public class And extends Performable {

    private final int NUM_OF_ARGS = 2;
    private final VarParser varParser;
    private Variable[] var;

    public And(VarParser varParser) {
        this.varParser = varParser;
    }

    public void perform(String msg) throws InvalidArgumentExeption {

        /* String[] args = StringSplitter.split(msg);
        Variable[] var = new Variable[NUM_OF_ARGS];
        for (int i = 0; i < var.length; i++) {
            var[i] = nameMap.getObgectFromName(args[i]);
        }*/
        var = varParser.getVar(msg, NUM_OF_ARGS);
        if (isValidTypes(var)) {

            var[0].setData(var[0].getData() & var[1].getData());
            CPU cpu = varParser.getCpu();
            cpu.setFlagState("OF", false);
            cpu.setFlagState("CF", false);

            if (var[0].getData() >= 0) cpu.setFlagState("SF", true);
            else cpu.setFlagState("SF", false);

            if (var[0].getData() == 0) cpu.setFlagState("ZF", true);
            else cpu.setFlagState("ZF", false);

            cpu.setFlagState("PF", Data.evenLowSetBits(Data.intToHex(var[0].getData(), 4)));
        } else {
            System.out.println(var[0].getType().toString() + " " + var[1].getType().toString());
            throw new InvalidArgumentExeption("Несоответствие типов");
        }
    }

    private boolean isValidTypes(Variable[] var) {
        if (isType0(VarTypes.reg8) && isType1(VarTypes.reg8))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.mem))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.mem))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.db))
            return true;
        return false;
    }

    private boolean isType0(VarTypes type) {
        return var[0].getType() == type;
    }

    private boolean isType1(VarTypes type) {
        return var[1].getType() == type;
    }
}
