package j8086emu.model.progcode.instructions;

import j8086emu.model.exeptions.InvalidArgumentExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.interfaces.Performable;
import j8086emu.model.interfaces.Variable;
import j8086emu.model.progcode.VarParser;
import j8086emu.model.progcode.types.VarTypes;

/**
 * @author vlad
 */
public class Div extends Performable {

    private final int NUM_OF_ARGS = 1;
    private final VarParser varParser;
    private final CPU cpu;
    private Variable[] var;

    public Div(VarParser varParser) {
        this.varParser = varParser;
        this.cpu = varParser.getCpu();
    }

    public void perform(String msg) throws InvalidArgumentExeption {
        // TODO Works correct with signs?

        var = varParser.getVar(msg, NUM_OF_ARGS);
        int argValue = var[0].getData();
        int dividend;
        if (argValue < 256) {
            dividend = cpu.getRegData("AX");
            cpu.setRegData("AL", dividend / argValue);
            cpu.setRegData("AH", dividend % argValue);
        } else {
            dividend = cpu.getRegData("DX") * 65536 + cpu.getRegData("AX");
            cpu.setRegData("AX", dividend / argValue);
            cpu.setRegData("DX", dividend % argValue);
        }
    }

    private boolean isValidTypes(Variable[] var) {
        if (isType0(VarTypes.reg16) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.reg16))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.mem))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.reg16) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.dw))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.db))
            return true;
        if (isType0(VarTypes.mem) && isType1(VarTypes.reg8))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.reg8))
            return true;
        if (isType0(VarTypes.reg8) && isType1(VarTypes.mem))
            return true;
        return false;
    }

    private boolean isType0(VarTypes type) {
        return var[0].getType() == type;
    }

    private boolean isType1(VarTypes type) {
        return var[1].getType() == type;
    }
}
