package j8086emu.view;

import j8086emu.model.hardware.CPU;
import j8086emu.model.hardware.registers.FlagNames;
import j8086emu.model.hardware.registers.RegNames;
import j8086emu.model.utils.Data;

import javax.swing.*;
import java.awt.event.*;

public class DataSetDialog extends JDialog {
    private JPanel contentPane;
    private JButton buttonCancel;
    private JComboBox comboBox1;
    private JTextField regDataTextField;
    private JButton regSetButton;
    private JComboBox comboBox2;
    private JTextField flagDataTextField;
    private JButton flagSetButton;
    private CPU cpu;
    private GUI gui;

    public DataSetDialog(GUI gui) {

        this.gui = gui;
        this.cpu = gui.getCpu();

        /*flagDataTextField.setText(cpu.getFlagState(comboBox2.getSelectedItem().toString()).toString());
        regDataTextField.setText(Data.intToHex(cpu.getRegData(comboBox1.getSelectedItem().toString()), 4));*/
        setContentPane(contentPane);
        setModal(true);

        buttonCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        });

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                onCancel();
            }
        });

// call onCancel() on ESCAPE
        contentPane.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onCancel();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    }

    private void onCancel() {
        setVisible(false);
    }

    private void createUIComponents() {
        comboBox1 = new JComboBox(RegNames.values());
        comboBox1.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                int i = cpu.getRegData(comboBox1.getSelectedItem().toString());
                regDataTextField.setText(Data.intToHex(i, 4));
            }
        });

        comboBox2 = new JComboBox(FlagNames.values());
        flagDataTextField = new JTextField();
        comboBox2.addItemListener(new ItemListener() {

            public void itemStateChanged(ItemEvent e) {
                Boolean i = cpu.getFlagState(((FlagNames) comboBox2.getSelectedItem()).toString());
                flagDataTextField.setText(i.toString());
            }
        });
        regSetButton = new JButton();
        regSetButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                regSetAction();


            }
        });

        flagSetButton = new JButton();
        flagSetButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String flagName = comboBox2.getSelectedItem().toString();
                cpu.setFlagState(flagName, !cpu.getFlagState(flagName));
                gui.setFlagData(flagName, cpu.getFlagState(flagName));
                flagDataTextField.setText(cpu.getFlagState(flagName).toString());
            }
        });
    }

    public void setVisible(boolean a) {
        if (a) {
            flagDataTextField.setText(cpu.getFlagState(comboBox2.getSelectedItem().toString()).toString());
            regDataTextField.setText(Data.intToHex(cpu.getRegData(comboBox1.getSelectedItem().toString()), 4));
        }
        super.setVisible(a);
    }

    private void regSetAction() {
        RegNames regName = (RegNames) comboBox1.getSelectedItem();
        //System.out.println(regName.toString());
        int data = 70000;
        try {
            data = Integer.parseInt(regDataTextField.getText(), 16);
        } catch (Exception e) {
            /*DialogWithText errorDialog = new DialogWithText("Проверьте правильность ввода значения для выбраного регистра");
            errorDialog.setSize(480, 240);
            errorDialog.setVisible(true);*/
            DialogWithText.showMessage("Проверьте правильность ввода значения для выбраного регистра");
            return;
        }
        if ((data > 65535) || (data < 0)) {
            /*DialogWithText errorDialog = new DialogWithText("Проверьте правильность ввода, значение вне допустимых пределов.");
            errorDialog.setSize(480, 240);
            errorDialog.setVisible(true);*/
            DialogWithText.showMessage("Проверьте правильность ввода, значение вне допустимых пределов.");
            return;
        }
        cpu.setRegData(regName.toString(), data);
        gui.setRegData(regName.toString(), cpu.getRegData(regName.toString()));

    }
}
