/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * GUI.java
 *
 * Created on 19.04.2010, 20:55:30
 */

package j8086emu.view;

import j8086emu.controller.ButtonsController;
import j8086emu.controller.RamController;
import j8086emu.model.exeptions.RamOutOfRangeExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.hardware.registers.FlagNames;
import j8086emu.model.hardware.registers.RegNames;
import j8086emu.model.interfaces.Widget;
import j8086emu.model.progcode.InstructionStructure;
import j8086emu.model.utils.Data;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Sone
 */
public class GUI extends javax.swing.JFrame {

    /**
     * Creates new form GUI
     */
    public GUI() {
        initComponents();
        initActions();
        initReg();
        initFlag();
        initListeners();
        initStack();
        getRamArea().setAutoscrolls(false);
    }

    private HashMap<String, JTextField> regTextFields;
    private HashMap<String, JLabel> flagLabels;
    private CPU cpu;
    //private CodeFacade codeFacade;
    private RamDataSetDialog ramDataSetDialog;
    private RamController ramController;
    private ButtonsController buttonsController;
    private JDialog dataSetDialog;
    private Screen screen;

    public CPU getCpu() {
        return cpu;
    }

    public JTextArea getRamArea() {
        return ramArea;
    }

    public JTextArea getStackArea() {
        return stackArea;
    }

    public void init(CPU cpu) {
        this.cpu = cpu;
        //this.codeFacade = cf;
        ramController = new RamController(this);
        ramController.createRamMap();
        ramDataSetDialog = new RamDataSetDialog(this, ramController);
        dataSetDialog = new DataSetDialog(this);
        HashMap widgetsMap = collectWidgets();
        buttonsController = new ButtonsController(codeTextArea, executeArea, widgetsMap, ramController, this);
        updateAllFlags();
        updateAllRegisters();
    }

    private void initStack() {
        stackArea.setLineWrap(true);
        stackArea.setWrapStyleWord(true);
    }

    private void initListeners() {
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                //To change body of implemented methods use File | Settings | File Templates.
            }
        });
        stepButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                programTabbedPane.setSelectedIndex(1);
                buttonsController.stepByStep();
                updateAllRegisters();
            }
        });
        launchButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                programTabbedPane.setSelectedIndex(1);
                buttonsController.runAllProgram();
                updateAllRegisters();
            }
        });
        compileButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                buttonsController.compile();
            }
        });
    }

    private void initActions() {
        exitMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        loadMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                loadItemActionPerformed(e);
            }
        });
        saveMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveItemActionPerformed(e);
            }
        });

    }

    private void loadItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openItemActionPerformed
        JFileChooser chooser = new JFileChooser("..");
        int rVal = chooser.showOpenDialog(this);
        String selectedFile = chooser.getCurrentDirectory().toString() + "/" + chooser.getSelectedFile().getName();
        if (rVal == JFileChooser.APPROVE_OPTION) {
            try {
                buttonsController.load(selectedFile);
            } catch (IOException ex) {
                Logger.getLogger(GUI8086emu.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_openItemActionPerformed

    private void saveItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveItemActionPerformed
        JFileChooser chooser = new JFileChooser("..");
        int rVal = chooser.showSaveDialog(this);
        String selectedFile = chooser.getCurrentDirectory().toString() + "/" + chooser.getSelectedFile().getName();
        if (rVal == JFileChooser.APPROVE_OPTION) {
            buttonsController.save(selectedFile);
        }
    }

    public JScrollPane getjScrollPane4() {
        return jScrollPane4;
    }

    public JTable getExecuteArea() {
        return executeArea;
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        textFieldIP = new javax.swing.JTextField();
        textFieldSS = new javax.swing.JTextField();
        textFieldES = new javax.swing.JTextField();
        textFieldDS = new javax.swing.JTextField();
        textFieldCS = new javax.swing.JTextField();
        textFieldSP = new javax.swing.JTextField();
        textFieldBP = new javax.swing.JTextField();
        textFieldDI = new javax.swing.JTextField();
        textFieldSI = new javax.swing.JTextField();
        textFieldDX = new javax.swing.JTextField();
        textFieldCX = new javax.swing.JTextField();
        textFieldBX = new javax.swing.JTextField();
        textFieldAX = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        flagZ = new javax.swing.JLabel();
        flagT = new javax.swing.JLabel();
        flagS = new javax.swing.JLabel();
        flagD = new javax.swing.JLabel();
        flagI = new javax.swing.JLabel();
        flagO = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jSeparator2 = new javax.swing.JSeparator();
        jLabel13 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        jLabel21 = new javax.swing.JLabel();
        jLabel22 = new javax.swing.JLabel();
        flagC = new javax.swing.JLabel();
        flagP = new javax.swing.JLabel();
        flagA = new javax.swing.JLabel();
        stepButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        launchButton = new javax.swing.JButton();
        compileButton = new javax.swing.JButton();
        programTabbedPane = new javax.swing.JTabbedPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        codeTextArea = new javax.swing.JTextArea();
        jScrollPane2 = new javax.swing.JScrollPane();
        executeArea = new javax.swing.JTable();
        jScrollPane3 = new javax.swing.JScrollPane();
        consoleTextArea = new javax.swing.JTextArea();
        jLabel32 = new javax.swing.JLabel();
        jScrollPane4 = new javax.swing.JScrollPane();
        ramArea = new javax.swing.JTextArea();
        jLabel33 = new javax.swing.JLabel();
        jScrollPane5 = new javax.swing.JScrollPane();
        stackArea = new javax.swing.JTextArea();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        saveMenuItem = new javax.swing.JMenuItem();
        loadMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        exitMenuItem = new javax.swing.JMenuItem();
        cpuMenu = new javax.swing.JMenu();
        resetMenuItem = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        autorsMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        setDataMenuItem = new javax.swing.JMenuItem();
        setRamMenuItem = new javax.swing.JMenuItem();
        debugMenu = new JMenu();
        clearBPMenuItem = new JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        textFieldIP.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldIP.setText("0001");

        textFieldSS.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldSS.setText("0002");

        textFieldES.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldES.setText("0003");

        textFieldDS.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldDS.setText("0004");

        textFieldCS.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldCS.setText("0005");

        textFieldSP.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldSP.setText("0006");

        textFieldBP.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldBP.setText("0007");

        textFieldDI.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldDI.setText("0008");

        textFieldSI.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldSI.setText("0009");

        textFieldDX.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldDX.setText("0010");

        textFieldCX.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldCX.setText("0011");

        textFieldBX.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldBX.setText("0012");

        textFieldAX.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        textFieldAX.setText("0013");

        textFieldAX.setEditable(false);
        textFieldBX.setEditable(false);
        textFieldCX.setEditable(false);
        textFieldDX.setEditable(false);
        textFieldBP.setEditable(false);
        textFieldCS.setEditable(false);
        textFieldDS.setEditable(false);
        textFieldDI.setEditable(false);
        textFieldES.setEditable(false);
        textFieldIP.setEditable(false);
        textFieldSI.setEditable(false);
        textFieldSP.setEditable(false);
        textFieldSS.setEditable(false);
        ramArea.setEditable(false);

        jLabel1.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel1.setText("AX");

        jLabel2.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel2.setText("BX");

        jLabel3.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel3.setText("CX");

        jLabel4.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel4.setText("DX");

        jLabel5.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel5.setText("SI");

        jLabel6.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel6.setText("DI");

        jLabel7.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel7.setText("BP");

        jLabel14.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel14.setText("SP");

        jLabel12.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel12.setText("CS");

        jLabel11.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel11.setText("DS");

        jLabel9.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel9.setText("ES");

        jLabel10.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel10.setText("SS");

        jLabel8.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel8.setText("IP");

        flagZ.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagZ.setText("Z");

        flagT.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagT.setText("T");

        flagS.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagS.setText("S");

        flagD.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagD.setText("D");

        flagI.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagI.setText("I");

        flagO.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagO.setText("O");

        jLabel15.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel15.setText("PF");

        jLabel13.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel13.setText("CF");

        jLabel16.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel16.setText("AF");

        jLabel17.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel17.setText("ZF");

        jLabel18.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel18.setText("SF");

        jLabel19.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel19.setText("TF");

        jLabel20.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel20.setText("IF");

        jLabel21.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel21.setText("DF");

        jLabel22.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel22.setText("OF");

        flagC.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagC.setText("C");

        flagP.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagP.setText("P");

        flagA.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        flagA.setText("A");

        stepButton.setText("Шаг");
        stepButton.setEnabled(false);

        stopButton.setText("Стоп");
        stopButton.setEnabled(false);

        launchButton.setText("Запуск");
        launchButton.setEnabled(false);

        compileButton.setText("Компиляция");

        codeTextArea.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        codeTextArea.setColumns(20);
        codeTextArea.setRows(5);
        codeTextArea.setText("Код");
        jScrollPane1.setViewportView(codeTextArea);

        programTabbedPane.addTab("Код", jScrollPane1);

        executeArea.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        /*executeArea.setModel(new javax.swing.table.DefaultTableModel(
                new Object[][]{
                        {new JCheckBox(), null, null, null},
                        {null, null, null, null}
                },
                new String[]{
                        "ТО", "Адрес", "Код", "HEX"
                }
        ));*/
        executeArea.setModel(new MyTableModel());
        //executeArea.setEnabled(false);
        jScrollPane2.setViewportView(executeArea);

        programTabbedPane.addTab("Выполнение", jScrollPane2);

        consoleTextArea.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        consoleTextArea.setColumns(20);
        consoleTextArea.setRows(5);
        consoleTextArea.setText("Консоль");
        jScrollPane3.setViewportView(consoleTextArea);

        jLabel32.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel32.setText("Память  | 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F ");

        ramArea.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        ramArea.setColumns(20);
        ramArea.setRows(5);
        ramArea.setText("Память");
        jScrollPane4.setViewportView(ramArea);


        jLabel33.setFont(new java.awt.Font("Monospaced", 1, 11)); // NOI18N
        jLabel33.setText("Стэк");

        stackArea.setColumns(20);
        stackArea.setRows(5);
        jScrollPane5.setViewportView(stackArea);

        debugMenu.setText("Отладка");

        clearBPMenuItem.setText("Сброс точек остановки");
        debugMenu.add(clearBPMenuItem);

        clearBPMenuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                for (int i = 0; i < executeArea.getRowCount(); i++) {
                    executeArea.setValueAt(new Boolean(false), i, 0);
                }
            }
        });


        fileMenu.setText("Файл");

        saveMenuItem.setText("Сохранить");
        fileMenu.add(saveMenuItem);

        loadMenuItem.setText("Загрузить");
        fileMenu.add(loadMenuItem);
        fileMenu.add(jSeparator1);

        exitMenuItem.setText("Выход");
        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);
        jMenuBar1.add(debugMenu);

        cpuMenu.setText("ЦПУ");

        resetMenuItem.setText("Сброс");
        resetMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cpu.reset();
                updateAllRegisters();
            }
        });
        cpuMenu.add(resetMenuItem);

        jMenuBar1.add(cpuMenu);

        editMenu.setText("Правка");

        setDataMenuItem.setText("Изменить значение регистров и флагов");
        setDataMenuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dataSetDialog.setBounds(200, 200, 480, 200);
                dataSetDialog.setVisible(true);
            }
        });
        editMenu.add(setDataMenuItem);

        setRamMenuItem.setText("Изменить значение ячеек памяти");
        setRamMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ramDataSetDialog.setBounds(200, 200, 480, 200);
                ramDataSetDialog.setVisible(true);
            }
        });

        editMenu.add(setRamMenuItem);

        jMenuBar1.add(editMenu);

        jMenu3.setText("Помошь");

        autorsMenuItem.setText("Авторы");
        jMenu3.add(autorsMenuItem);

        jMenuBar1.add(jMenu3);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addGroup(layout.createSequentialGroup()
                                        .addGap(10, 10, 10)
                                        .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 786, Short.MAX_VALUE))
                                .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(jLabel1)
                                                        .addComponent(jLabel2)
                                                        .addComponent(jLabel3)
                                                        .addComponent(jLabel4)
                                                        .addComponent(jLabel5)
                                                        .addComponent(jLabel6)
                                                        .addComponent(jLabel7)
                                                        .addComponent(jLabel14)
                                                        .addComponent(jLabel12)
                                                        .addComponent(jLabel11)
                                                        .addComponent(jLabel9)
                                                        .addComponent(jLabel10)
                                                        .addComponent(jLabel8))
                                                .addGap(6, 6, 6)
                                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(textFieldSI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldCS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldDS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldDX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldBP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldCX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldDI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldAX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldSP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldBX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldIP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldSS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(textFieldES, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                        .addGroup(layout.createSequentialGroup()
                                                .addGap(20, 20, 20)
                                                .addComponent(flagC))
                                        .addComponent(jLabel13)
                                        .addComponent(jSeparator2, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel15)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagP))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel16)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagA))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel17)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagZ))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel18)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagS))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel19)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagT))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel20)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagI))
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(jLabel21)
                                                .addGap(6, 6, 6)
                                                .addComponent(flagD))
                                        .addGroup(layout.createSequentialGroup()
                                        .addComponent(jLabel22)
                                        .addGap(6, 6, 6)
                                        .addComponent(flagO)))
                                .addGap(6, 6, 6)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(layout.createSequentialGroup()
                                                .addComponent(compileButton)
                                                .addGap(6, 6, 6)
                                                .addComponent(launchButton)
                                                .addGap(6, 6, 6)
                                                .addComponent(stopButton)
                                                .addGap(6, 6, 6)
                                                .addComponent(stepButton))
                                        .addComponent(programTabbedPane, 0, 0, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 433, Short.MAX_VALUE)
                                .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 433, Short.MAX_VALUE)
                                .addComponent(jLabel32)
                                .addComponent(jLabel33))))
                        .addContainerGap())
        );
        layout.setVerticalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(layout.createSequentialGroup()
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addGroup(layout.createSequentialGroup()
                                                        .addComponent(jLabel1)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel2)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel3)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel4)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel5)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel6)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel7)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel14)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel12)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel11)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel9)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel10)
                                                        .addGap(4, 4, 4)
                                                        .addComponent(jLabel8))
                                                .addGroup(layout.createSequentialGroup()
                                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(80, 80, 80)
                                                                        .addComponent(textFieldSI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(160, 160, 160)
                                                                        .addComponent(textFieldCS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(180, 180, 180)
                                                                        .addComponent(textFieldDS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(60, 60, 60)
                                                                        .addComponent(textFieldDX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(120, 120, 120)
                                                                        .addComponent(textFieldBP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(40, 40, 40)
                                                                        .addComponent(textFieldCX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(100, 100, 100)
                                                                        .addComponent(textFieldDI, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addComponent(textFieldAX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                                .addGroup(layout.createSequentialGroup()
                                                                        .addGap(140, 140, 140)
                                                                        .addComponent(textFieldSP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addGroup(layout.createSequentialGroup()
                                                                .addGap(20, 20, 20)
                                                                .addComponent(textFieldBX, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                                        .addGap(18, 18, 18)
                                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addGroup(layout.createSequentialGroup()
                                                                .addGap(20, 20, 20)
                                                                .addComponent(textFieldIP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                        .addComponent(textFieldSS, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                                .addGroup(layout.createSequentialGroup()
                                                .addGap(200, 200, 200)
                                                .addComponent(textFieldES, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                        .addGap(6, 6, 6)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(flagC)
                                                .addComponent(jLabel13)
                                                .addComponent(jSeparator2, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel15)
                                                .addComponent(flagP))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel16)
                                                .addComponent(flagA))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel17)
                                                .addComponent(flagZ))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel18)
                                                .addComponent(flagS))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel19)
                                                .addComponent(flagT))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel20)
                                                .addComponent(flagI))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jLabel21)
                                                .addComponent(flagD))
                                        .addGap(4, 4, 4)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(jLabel22)
                                        .addComponent(flagO)))
                                .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(compileButton)
                                        .addComponent(launchButton)
                                        .addComponent(stopButton)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(stepButton)
                                        .addComponent(jLabel32)))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addGroup(layout.createSequentialGroup()
                                        .addComponent(jScrollPane4, javax.swing.GroupLayout.PREFERRED_SIZE, 313, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jLabel33)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jScrollPane5, javax.swing.GroupLayout.DEFAULT_SIZE, 105, Short.MAX_VALUE))
                                .addComponent(programTabbedPane, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 444, Short.MAX_VALUE))))
                        .addGap(11, 11, 11)
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents


    /*public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new GUI().setVisible(true);
            }
        });
    }*/

    public Screen getScreen() {
        if (screen == null) {
            screen = new Screen();
        }
        return screen;
    }

    private void initReg() {
        regTextFields = new HashMap<String, JTextField>();
        regTextFields.put("AX", textFieldAX);
        regTextFields.put("BX", textFieldBX);
        regTextFields.put("CX", textFieldCX);
        regTextFields.put("DX", textFieldDX);
        regTextFields.put("SI", textFieldSI);
        regTextFields.put("DI", textFieldDI);
        regTextFields.put("BP", textFieldBP);
        regTextFields.put("SP", textFieldSP);
        regTextFields.put("CS", textFieldCS);
        regTextFields.put("ES", textFieldES);
        regTextFields.put("DS", textFieldDS);
        regTextFields.put("SS", textFieldSS);
        regTextFields.put("IP", textFieldIP);
    }

    private void initFlag() {
        flagLabels = new HashMap<String, JLabel>();
        flagLabels.put("CF", flagC);
        flagLabels.put("PF", flagP);
        flagLabels.put("AF", flagA);
        flagLabels.put("ZF", flagZ);
        flagLabels.put("SF", flagS);
        flagLabels.put("TF", flagT);
        flagLabels.put("IF", flagI);
        flagLabels.put("DF", flagD);
        flagLabels.put("OF", flagO);

    }

    private HashMap collectWidgets() {
        HashMap<String, Widget> widgetsMap = new HashMap<String, Widget>();
        widgetsMap.put("AX", new TextWidget(textFieldAX));
        widgetsMap.put("BX", new TextWidget(textFieldBX));
        widgetsMap.put("CX", new TextWidget(textFieldCX));
        widgetsMap.put("DX", new TextWidget(textFieldDX));
        widgetsMap.put("SI", new TextWidget(textFieldSI));
        widgetsMap.put("DI", new TextWidget(textFieldDI));
        widgetsMap.put("BP", new TextWidget(textFieldBP));
        widgetsMap.put("SP", new TextWidget(textFieldSP));
        widgetsMap.put("CS", new TextWidget(textFieldCS));
        widgetsMap.put("ES", new TextWidget(textFieldES));
        widgetsMap.put("DS", new TextWidget(textFieldDS));
        widgetsMap.put("SS", new TextWidget(textFieldSS));
        widgetsMap.put("IP", new TextWidget(textFieldIP));

//        widgetsMap.put("FLAGS", new FlagsWidget(flagLabels));
        return widgetsMap;
    }

    private void updateAllRegisters() {
        for (RegNames regName : RegNames.values()) {
            setRegData(regName.toString(), cpu.getRegData(regName.toString()));
        }
    }

    public void updateRam(int startAdress, int endAdress) {
        try {
            ramController.updateRamMap(startAdress, endAdress);
        } catch (RamOutOfRangeExeption ramOutOfRangeExeption) {
            ramOutOfRangeExeption.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    private void updateAllFlags() {
        for (FlagNames flagName : FlagNames.values()) {
            setFlagData(flagName.toString(), cpu.getFlagState(flagName.toString()));
        }
    }

    void setRegData(String name, Integer data) {
        regTextFields.get(name).setText(Data.intToHex(data, 4).toUpperCase());
    }

    public void setFlagData(String name, Boolean data) {
        if (!data)
            flagLabels.get(name).setText("0");
        else
            flagLabels.get(name).setText("1");
    }

    public void reset() {
        updateAllFlags();
        updateAllRegisters();
        updateRam(0, 65535);
    }

    public void setEnabledExecutionButtons(boolean isEnabled) {
        launchButton.setEnabled(isEnabled);
        stepButton.setEnabled(isEnabled);
    }

    public void showErrorWarningMessages(ArrayList<String> errorMessages, ArrayList<String> warningMessages) {
        consoleTextArea.setText("");
        if (errorMessages != null) {
            for (String errorMsg : errorMessages) {
                consoleTextArea.append(errorMsg + "\n");
            }
        }
        if (warningMessages != null) {
            for (String warningMsg : warningMessages) {
                consoleTextArea.append(warningMsg + "\n");
            }
        }
    }

    public void showMachineInstructions(ArrayList<InstructionStructure> instructions) {
        executeArea.selectAll();
        executeArea.clearSelection();
        for (InstructionStructure structure : instructions) {
            //executeArea.append(structure.getOffset() + " " + structure.getHexInstruction() + " " + structure.isData() + "\n");
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton compileButton;
    private javax.swing.JButton launchButton;
    private javax.swing.JButton stopButton;
    private javax.swing.JButton stepButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel flagC;
    private javax.swing.JLabel flagP;
    private javax.swing.JLabel flagA;
    private javax.swing.JLabel flagZ;
    private javax.swing.JLabel flagS;
    private javax.swing.JLabel flagT;
    private javax.swing.JLabel flagI;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel flagD;
    private javax.swing.JLabel flagO;
    private javax.swing.JLabel jLabel32;
    private javax.swing.JLabel jLabel33;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenu debugMenu;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu cpuMenu;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem clearBPMenuItem;
    private javax.swing.JMenuItem saveMenuItem;
    private javax.swing.JMenuItem loadMenuItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenuItem resetMenuItem;
    private javax.swing.JMenuItem autorsMenuItem;
    private javax.swing.JMenuItem setDataMenuItem;
    private javax.swing.JMenuItem setRamMenuItem;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JTabbedPane programTabbedPane;
    private javax.swing.JTable executeArea;
    private javax.swing.JTextArea codeTextArea;
    private javax.swing.JTextArea consoleTextArea;
    private javax.swing.JTextArea ramArea;
    private javax.swing.JTextArea stackArea;
    private javax.swing.JTextField textFieldAX;
    private javax.swing.JTextField textFieldDS;
    private javax.swing.JTextField textFieldES;
    private javax.swing.JTextField textFieldSS;
    private javax.swing.JTextField textFieldIP;
    private javax.swing.JTextField textFieldBX;
    private javax.swing.JTextField textFieldCX;
    private javax.swing.JTextField textFieldDX;
    private javax.swing.JTextField textFieldSI;
    private javax.swing.JTextField textFieldDI;
    private javax.swing.JTextField textFieldBP;
    private javax.swing.JTextField textFieldSP;
    private javax.swing.JTextField textFieldCS;
    // End of variables declaration//GEN-END:variables

    class MyTableModel extends AbstractTableModel {
        private String[] columnNames = {"ТО", "Адрес", "Код", "HEX"};
        private Object[][] data = {
                {new Boolean(true), "", "", ""}
        };

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return data.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            return data[row][col];
        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */

        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * Don't need to implement this method unless your table's
         * editable.
         */

        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears onscreen.
            if (col > 0) {
                return false;
            } else {
                return true;
            }
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */

        public void setValueAt(Object value, int row, int col) {
            data[row][col] = value;
            fireTableCellUpdated(row, col);
        }

    }

}
