package j8086emu.view;

import j8086emu.controller.ButtonsController;
import j8086emu.controller.RamController;
import j8086emu.model.exeptions.RamOutOfRangeExeption;
import j8086emu.model.hardware.CPU;
import j8086emu.model.hardware.registers.FlagNames;
import j8086emu.model.hardware.registers.RegNames;
import j8086emu.model.interfaces.Widget;
import j8086emu.model.progcode.CodeFacade;
import j8086emu.model.progcode.InstructionStructure;
import j8086emu.model.utils.Data;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * J8086emu
 * User: Sone
 * Date: 09.03.2010
 * Time: 14:16:26
 */
public class GUI8086emu extends JFrame {
    private JButton stopButton;
    public JPanel panel1;
    private JButton launchButton;
    private JButton stepButton;
    private JTabbedPane programTabbedPane;
    private JButton compileButton;
    private JLabel flagC;
    private JLabel flagP;
    private JLabel flagA;
    private JLabel flagZ;
    private JLabel flagS;
    private JLabel flagT;
    private JLabel flagI;
    private JLabel flagD;
    private JLabel flagO;
    private JMenuBar menuBar;
    private JTextArea codeTextArea;
    private JTextArea ramArea;
    private JTextArea stackArea;
    private JTextField textFieldAX;
    private JTextField textFieldBX;
    private JTextField textFieldCX;
    private JTextField textFieldDX;
    private JTextField textFieldSI;
    private JTextField textFieldDI;
    private JTextField textFieldBP;
    private JTextField textFieldSP;
    private JTextField textFieldCS;
    private JTextField textFieldES;
    private JTextField textFieldDS;
    private JTextField textFieldSS;
    private JTextField textFieldIP;
    private JTextArea executionTextArea;
    private JPanel pointsPanel;
    private HashMap<String, JTextField> regTextFields;
    private HashMap<String, JLabel> flagLabels;
    private CPU cpu;
    private CodeFacade codeFacade;
    private RamDataSetDialog ramDataSetDialog;
    private RamController ramController;
    private ButtonsController buttonsController;
    private JDialog dataSetDialog;

    private JTextArea errorArea;

    public Screen getScreen() {
        if (screen == null) {
            screen = new Screen();
        }
        return screen;
    }

    private Screen screen;

    public GUI8086emu() {
        initReg();
        initFlag();
        initListeners();
        initStack();
        //codeTextArea.setText("seg segment\n\nseg ends");
    }

    public CPU getCpu() {
        return cpu;
    }

    /*public void init(CPU cpu, CodeFacade cf) {
        this.cpu = cpu;
        this.codeFacade = cf;
        ramController = new RamController(this);
        ramController.createRamMap();
        ramDataSetDialog = new RamDataSetDialog(this, ramController);
        dataSetDialog = new DataSetDialog(this);
        HashMap widgetsMap = collectWidgets();
        buttonsController = new ButtonsController(codeTextArea, executionTextArea, widgetsMap, ramController, this);
        updateAllFlags();
        updateAllRegisters();
    }*/

    void setRegData(String name, Integer data) {
        regTextFields.get(name).setText(Data.intToHex(data, 4).toUpperCase());
    }

    private void initReg() {
        regTextFields = new HashMap<String, JTextField>();
        regTextFields.put("AX", textFieldAX);
        regTextFields.put("BX", textFieldBX);
        regTextFields.put("CX", textFieldCX);
        regTextFields.put("DX", textFieldDX);
        regTextFields.put("SI", textFieldSI);
        regTextFields.put("DI", textFieldDI);
        regTextFields.put("BP", textFieldBP);
        regTextFields.put("SP", textFieldSP);
        regTextFields.put("CS", textFieldCS);
        regTextFields.put("ES", textFieldES);
        regTextFields.put("DS", textFieldDS);
        regTextFields.put("SS", textFieldSS);
        regTextFields.put("IP", textFieldIP);
    }

    private void initFlag() {
        flagLabels = new HashMap<String, JLabel>();
        flagLabels.put("CF", flagC);
        flagLabels.put("PF", flagP);
        flagLabels.put("AF", flagA);
        flagLabels.put("ZF", flagZ);
        flagLabels.put("SF", flagS);
        flagLabels.put("TF", flagT);
        flagLabels.put("IF", flagI);
        flagLabels.put("DF", flagD);
        flagLabels.put("OF", flagO);

    }

    public void setFlagData(String name, Boolean data) {
        if (!data)
            flagLabels.get(name).setText("0");
        else
            flagLabels.get(name).setText("1");
    }

    private void createUIComponents() {
        initMenuBar();
        //initCodeArea();
    }

    private void initMenuBar() {
        menuBar = new JMenuBar();
        JMenu file = new JMenu("Файл");
        JMenuItem save = new JMenuItem("Сохранить");
        JMenuItem load = new JMenuItem("Загрузить");
        JMenuItem exit = new JMenuItem("Выход");
        exit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        load.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                loadItemActionPerformed(e);
            }
        });
        save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveItemActionPerformed(e);
            }
        });
        file.add(save);
        file.add(load);
        file.add(new JSeparator());
        file.add(exit);
        menuBar.add(file);
        JMenu edit = new JMenu("Правка");
        JMenuItem setRamCell = new JMenuItem("Изменить значение ячейки памяти");
        setRamCell.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ramDataSetDialog.setBounds(200, 200, 480, 200);
                ramDataSetDialog.setVisible(true);
            }
        });
        JMenuItem setRegFlagData = new JMenuItem("Изменить содержимое регистров и флагов");
        setRegFlagData.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                dataSetDialog.setBounds(200, 200, 480, 200);
                dataSetDialog.setVisible(true);
            }
        });
        edit.add(setRamCell);
        edit.add(setRegFlagData);
        menuBar.add(edit);
        JMenu code = new JMenu("ЦПУ");
        JMenuItem compile = new JMenuItem("Сброс");
        compile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cpu.reset();
                updateAllRegisters();
            }
        });
        code.add(compile);
        menuBar.add(code);
        JMenu help = new JMenu("Помощь");
        menuBar.add(help);
    }

    private void initListeners() {
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                //To change body of implemented methods use File | Settings | File Templates.
            }
        });
        stepButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                programTabbedPane.setSelectedIndex(1);
                buttonsController.stepByStep();
                updateAllRegisters();
            }
        });
        launchButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                programTabbedPane.setSelectedIndex(1);
                buttonsController.runAllProgram();
                updateAllRegisters();
            }
        });
        compileButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                buttonsController.compile();
            }
        });
    }

    private void loadItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openItemActionPerformed
        JFileChooser chooser = new JFileChooser("..");
        int rVal = chooser.showOpenDialog(this);
        String selectedFile = chooser.getCurrentDirectory().toString() + "/" + chooser.getSelectedFile().getName();
        if (rVal == JFileChooser.APPROVE_OPTION) {
            try {
                buttonsController.load(selectedFile);
            } catch (IOException ex) {
                Logger.getLogger(GUI8086emu.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_openItemActionPerformed

    private void saveItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveItemActionPerformed
        JFileChooser chooser = new JFileChooser("..");
        int rVal = chooser.showSaveDialog(this);
        String selectedFile = chooser.getCurrentDirectory().toString() + "/" + chooser.getSelectedFile().getName();
        if (rVal == JFileChooser.APPROVE_OPTION) {
            buttonsController.save(selectedFile);
        }
    }

    private void updateAllRegisters() {
        for (RegNames regName : RegNames.values()) {
            setRegData(regName.toString(), cpu.getRegData(regName.toString()));
        }
    }

    public void updateRam(int startAdress, int endAdress) {
        try {
            ramController.updateRamMap(startAdress, endAdress);
        } catch (RamOutOfRangeExeption ramOutOfRangeExeption) {
            ramOutOfRangeExeption.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }

    private void updateAllFlags() {
        for (FlagNames flagName : FlagNames.values()) {
            setFlagData(flagName.toString(), cpu.getFlagState(flagName.toString()));
        }
    }

    public JTextArea getRamArea() {
        return ramArea;
    }

    public JTextArea getStackArea() {
        return stackArea;
    }

    private void initStack() {
        stackArea.setLineWrap(true);
        stackArea.setWrapStyleWord(true);
    }

    private HashMap collectWidgets() {
        HashMap<String, Widget> widgetsMap = new HashMap<String, Widget>();
        widgetsMap.put("AX", new TextWidget(textFieldAX));
        widgetsMap.put("BX", new TextWidget(textFieldBX));
        widgetsMap.put("CX", new TextWidget(textFieldCX));
        widgetsMap.put("DX", new TextWidget(textFieldDX));
        widgetsMap.put("SI", new TextWidget(textFieldSI));
        widgetsMap.put("DI", new TextWidget(textFieldDI));
        widgetsMap.put("BP", new TextWidget(textFieldBP));
        widgetsMap.put("SP", new TextWidget(textFieldSP));
        widgetsMap.put("CS", new TextWidget(textFieldCS));
        widgetsMap.put("ES", new TextWidget(textFieldES));
        widgetsMap.put("DS", new TextWidget(textFieldDS));
        widgetsMap.put("SS", new TextWidget(textFieldSS));
        widgetsMap.put("IP", new TextWidget(textFieldIP));

//        widgetsMap.put("FLAGS", new FlagsWidget(flagLabels));
        return widgetsMap;
    }

    /*private void initCodeArea() {
        codeTextArea = new JTextArea();
    }*/

    public void reset() {
        updateAllFlags();
        updateAllRegisters();
        updateRam(0, 256); //todo: update area
    }

    public void setEnabledExecutionButtons(boolean isEnabled) {
        launchButton.setEnabled(isEnabled);
        stepButton.setEnabled(isEnabled);
    }

    public void showScreen() {
        if (screen == null) {
            screen = new Screen();
        }
        screen.setVisible(true);
    }

    public void showErrorWarningMessages(ArrayList<String> errorMessages, ArrayList<String> warningMessages) {
        //errorArea.setText("");            //TODO: decomment after attaching new view
        if (errorMessages != null) {
            for (String errorMsg : errorMessages) {
                //errorArea.append(errorMsg + "\n");
            }
        }
        if (warningMessages != null) {
            for (String warningMsg : warningMessages) {
                //errorArea.append(warningMsg + "\n");
            }
        }
    }
    public void showMachineInstructions(ArrayList<InstructionStructure> instructions) {
        executionTextArea.setText("");
        for(InstructionStructure structure : instructions) {
            executionTextArea.append(structure.getOffset() + " " + structure.getHexInstruction() + " " + structure.isData() + "\n");
        }
    }
}
